# This file is Copyright (c) 2015, 2017 William D. Jones <thor0505@comcast.net>
# License: BSD

from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform
from migen.build.xilinx.programmer import XC3SProg


_io = [
    ("clk50", 0, Pins("P43"), IOStandard("LVCMOS33")),

    ("user_btn", 0, Pins("P41"), IOStandard("LVTTL")),

    # The serial interface and flash memory have a shared SPI bus.
    # FPGA is secondary
    ("spiserial", 0,
        Subsignal("cs_n", Pins("P39"), IOStandard("LVTTL")),
        Subsignal("clk", Pins("P53"), IOStandard("LVTTL")),
        Subsignal("mosi", Pins("P46"), IOStandard("LVTTL")),
        Subsignal("miso", Pins("P51"), IOStandard("LVTTL"))
    ),

    # FPGA is primary
    ("spiflash", 0,
        Subsignal("cs_n", Pins("P27"), IOStandard("LVTTL")),
        Subsignal("clk", Pins("P53"), IOStandard("LVTTL")),
        Subsignal("mosi", Pins("P46"), IOStandard("LVTTL")),
        Subsignal("miso", Pins("P51"), IOStandard("LVTTL"))
    ),

    ("spiflash2x", 0,
        Subsignal("cs_n", Pins("P27")),
        Subsignal("clk", Pins("P53")),
        Subsignal("dq", Pins("P46", "P51")),
        IOStandard("LVTTL"), Misc("SLEW=FAST")
    ),

    # ADC over SPI- FPGA is primary
    ("adc", 0,
        Subsignal("cs_n", Pins("P12"), IOStandard("LVTTL")),
        Subsignal("clk",  Pins("P9"), IOStandard("LVTTL")),
        Subsignal("mosi", Pins("P10"), IOStandard("LVTTL")),
        Subsignal("miso", Pins("P21"), IOStandard("LVTTL"))
    ),

    # GPIO control- SRAM and connectors are shared: these pins control how
    # to access each. Recommended to combine with gpio_sram_bus extension,
    # since these pins are related but not exposed on connectors.
    ("gpio_ctl", 0,
        Subsignal("ce_n", Pins("P3")),  # Memory chip-enable. Called MEM_CEN
        # in schematic.
        Subsignal("bussw_oe_n", Pins("P30")),  # 5V tolerant GPIO is shared
        # w/ memory using this pin.
        IOStandard("LVTTL"), Misc("SLEW=FAST")
    )
]

# Perhaps define some connectors as having a specific purpose- i.e. a 5V GPIO
# bus with data, peripheral-select, and control signals?
_connectors = [
    ("GPIO", """P59 P60 P61 P62 P64 P57
                P56 P52 P50 P49 P85 P84
                P83 P78 P77 P65 P70 P71
                P72 P73 P5 P4 P6 P98
                P94 P93 P90 P89 P88 P86"""),  # 5V I/O- LVTTL
    ("DIO", "P20 P32 P33 P34 P35 P36 P37"),  # Fast 3.3V IO (Directly attached
    # to FPGA)- LVCMOS33
    ("CLKIO", "P40 P44"),  # Clock IO (Can be used as GPIO)- LVCMOS33
    ("INPUT", "P68 P97 P7 P82"),  # Input-only pins- LVCMOS33
    ("LED", "P13 P15 P16 P19"),  # LEDs can be used as pins as well- LVTTL.
    ("PMOD", "P5 P4 P6 P98 P94 P93 P90 P89")  # Baseboard PMOD.
    # Overlaps w/ GPIO bus.
]


# Some default useful extensions- use platform.add_extension() to use, e.g.
# from migen.build.platforms import mercury
# plat = mercury.Platform()
# plat.add_extension(mercury.gpio_sram)

# SRAM and 5V-tolerant I/O share a parallel bus on 200k gate version. The SRAM
# controller needs to take care of switching the bus between the two. Meant to
# be Cat() into one GPIO bus, and combined with gpio_ctl.
gpio_sram = [
    ("gpio_sram_bus", 0,
        Subsignal("a", Pins("""GPIO:0 GPIO:1 GPIO:2 GPIO:3
                               GPIO:4 GPIO:5 GPIO:6 GPIO:7
                               GPIO:8 GPIO:9 GPIO:10 GPIO:11
                               GPIO:12 GPIO:13 GPIO:14 GPIO:15
                               GPIO:16 GPIO:17 GPIO:18 GPIO:19""")),
        # A19 is actually unused- free for GPIO
        # 8-bit data bus
        Subsignal("d", Pins("""GPIO:20 GPIO:21 GPIO:22 GPIO:23
                               GPIO:24 GPIO:25 GPIO:26 GPIO:27""")),
        Subsignal("we_n", Pins("GPIO:28")),
        Subsignal("unused", Pins("GPIO:29")),  # Only used by GPIO.
        # Subsignal("oe_n", Pins()),  # If OE wasn't tied to ground on Mercury,
        # this pin would be here.
        IOStandard("LVTTL"), Misc("SLEW=FAST")
    )
]

# The "serial port" is in fact over SPI. The creators of the board provide a
# VHDL file for talking over this interface. In light of space constraints and
# the fact that both the FT245RL and FPGA can BOTH be SPI primaries, however,
# it may be necessary to sacrifice two "high-speed" (DIO, INPUT) pins instead.
serial = [
    ("serial", 0,
        Subsignal("tx", Pins("DIO:0"), IOStandard("LVCMOS33")),  # FTDI D1
        Subsignal("rx", Pins("INPUT:0"), IOStandard("LVCMOS33"))  # FTDI D0
    )
]

leds = [
    ("user_led", 0, Pins("LED:0"), IOStandard("LVTTL")),
    ("user_led", 1, Pins("LED:1"), IOStandard("LVTTL")),
    ("user_led", 2, Pins("LED:2"), IOStandard("LVTTL")),
    ("user_led", 3, Pins("LED:3"), IOStandard("LVTTL"))
]


# The remaining peripherals only make sense w/ the Baseboard installed.
# See: http://www.micro-nova.com/mercury-baseboard/
sw = [
    ("sw", 0, Pins("GPIO:0"), IOStandard("LVTTL")),
    ("sw", 1, Pins("GPIO:1"), IOStandard("LVTTL")),
    ("sw", 2, Pins("GPIO:2"), IOStandard("LVTTL")),
    ("sw", 3, Pins("GPIO:3"), IOStandard("LVTTL")),
    ("sw", 4, Pins("GPIO:4"), IOStandard("LVTTL")),
    ("sw", 5, Pins("GPIO:5"), IOStandard("LVTTL")),
    ("sw", 6, Pins("GPIO:6"), IOStandard("LVTTL")),
    ("sw", 7, Pins("GPIO:7"), IOStandard("LVTTL"))
]

user_btn = [
    ("user_btn", 1, Pins("INPUT:0"), IOStandard("LVTTL")),
    ("user_btn", 2, Pins("INPUT:1"), IOStandard("LVTTL")),
    ("user_btn", 3, Pins("INPUT:2"), IOStandard("LVTTL")),
    ("user_btn", 4, Pins("INPUT:3"), IOStandard("LVTTL"))
]

vga = [
    ("vga_out", 0,
        Subsignal("hsync_n", Pins("LED:2"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST")),
        Subsignal("vsync_n", Pins("LED:3"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST")),
        Subsignal("r", Pins("DIO:0 DIO:1 DIO:2"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST")),
        Subsignal("g", Pins("DIO:3 DIO:4 DIO:5"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST")),
        Subsignal("b", Pins("DIO:6 CLKIO:0"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST"))
    )
]

extclk = [
    ("extclk", 0, Pins("CLKIO:1"), IOStandard("LVCMOS33"))
]

sevenseg = [
    ("sevenseg", 0,
        Subsignal("segment7", Pins("GPIO:12"), IOStandard("LVTTL")),  # A
        Subsignal("segment6", Pins("GPIO:13"), IOStandard("LVTTL")),  # B
        Subsignal("segment5", Pins("GPIO:14"), IOStandard("LVTTL")),  # C
        Subsignal("segment4", Pins("GPIO:15"), IOStandard("LVTTL")),  # D
        Subsignal("segment3", Pins("GPIO:16"), IOStandard("LVTTL")),  # E
        Subsignal("segment2", Pins("GPIO:17"), IOStandard("LVTTL")),  # F
        Subsignal("segment1", Pins("GPIO:18"), IOStandard("LVTTL")),  # G
        Subsignal("segment0", Pins("GPIO:19"), IOStandard("LVTTL")),  # Dot
        Subsignal("enable0", Pins("GPIO:8"), IOStandard("LVTTL")),   # EN0
        Subsignal("enable1", Pins("GPIO:9"), IOStandard("LVTTL")),   # EN1
        Subsignal("enable2", Pins("GPIO:10"), IOStandard("LVTTL")),  # EN2
        Subsignal("enable3", Pins("GPIO:11"), IOStandard("LVTTL"))  # EN2
    )
]

ps2 = [
    ("ps2", 0,
        Subsignal("clk", Pins("LED:1"), IOStandard("LVCMOS33")),
        Subsignal("data", Pins("LED:0"), IOStandard("LVCMOS33"))
    )
]

audio = [
    ("audio", 0,
        Subsignal("l", Pins("GPIO:29"), IOStandard("LVTTL")),
        Subsignal("r", Pins("GPIO:28"), IOStandard("LVTTL"))
    )
]


class Platform(XilinxPlatform):
    default_clk_name = "clk50"
    default_clk_period = 20

    def __init__(self, device="xc3s200a-4-vq100"):
        XilinxPlatform.__init__(self, device, _io, _connectors)
        # Small device- optimize for AREA instead of SPEED (LM32 runs at about
        # 60-65MHz in AREA configuration).
        self.toolchain.xst_opt = """-ifmt MIXED
-use_new_parser yes
-opt_mode AREA
-register_balancing yes"""

    def create_programmer(self):
        raise NotImplementedError



from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform


_io = [
        ("user_btn", 0, Pins("V4"), IOStandard("LVCMOS33"),
            Misc("PULLDOWN"), Misc("TIG")),

        ("user_led", 0, Pins("P4"), Misc("SLEW=QUIETIO"), IOStandard("LVCMOS18")),
        ("user_led", 1, Pins("L6"), Misc("SLEW=QUIETIO"), IOStandard("LVCMOS18")),
        ("user_led", 2, Pins("F5"), Misc("SLEW=QUIETIO"), IOStandard("LVCMOS18")),
        ("user_led", 3, Pins("C2"), Misc("SLEW=QUIETIO"), IOStandard("LVCMOS18")),

        ("user_dip", 0, Pins("B3"), Misc("PULLDOWN"), IOStandard("LVCMOS33")),
        ("user_dip", 1, Pins("A3"), Misc("PULLDOWN"), IOStandard("LVCMOS33")),
        ("user_dip", 2, Pins("B4"), Misc("PULLDOWN"), IOStandard("LVCMOS33")),
        ("user_dip", 3, Pins("A4"), Misc("PULLDOWN"), IOStandard("LVCMOS33")),

        # TI CDCE913 programmable triple-output PLL
        ("clk_y1", 0, Pins("V10"), IOStandard("LVCMOS33")),  # default: 40 MHz
        ("clk_y2", 0, Pins("K15"), IOStandard("LVCMOS33")),  # default: 66 2/3 MHz
        ("clk_y3", 0, Pins("C10"), IOStandard("LVCMOS33")),  # default: 100 MHz

        # Maxim DS1088LU oscillator, not populated
        ("clk_backup", 0, Pins("R8"), IOStandard("LVCMOS33")),

        # TI CDCE913 PLL I2C control
        ("pll", 0,
            Subsignal("scl", Pins("P12")),
            Subsignal("sda", Pins("U13")),
            Misc("PULLUP"),
            IOStandard("LVCMOS33")),

        # Micron N25Q128 SPI Flash
        ("spiflash", 0,
            Subsignal("clk", Pins("R15")),
            Subsignal("cs_n", Pins("V3")),
            Subsignal("dq", Pins("T13 R13 T14 V14")),
            IOStandard("LVCMOS33")),

        # PMOD extension connectors
        ("pmod", 0,
            Subsignal("d", Pins("F15 F16 C17 C18 F14 G14 D17 D18")),
            IOStandard("LVCMOS33")),
        ("pmod", 1,
            Subsignal("d", Pins("H12 G13 E16 E18 K12 K13 F17 F18")),
            IOStandard("LVCMOS33")),

        ("pmod_diff", 0,
            Subsignal("io", Pins("F15 C17 F14 D17 H12 E16 K12 F17")),
            Subsignal("iob", Pins("F16 C18 G14 D18 G13 E18 K13 F18")),
            IOStandard("LVCMOS33")),

        ("serial", 0,
            Subsignal("tx", Pins("T7"), Misc("SLEW=SLOW")),
            Subsignal("rx", Pins("R7"), Misc("PULLUP")),
            IOStandard("LVCMOS33")),

        ("ddram_clock", 0,
            Subsignal("p", Pins("G3")),
            Subsignal("n", Pins("G1")),
            IOStandard("MOBILE_DDR")),  # actually DIFF_

        # Micron MT46H32M16LFBF-5 LPDDR
        ("ddram", 0,
            Subsignal("a", Pins("J7 J6 H5 L7 F3 H4 H3 H6 "
                "D2 D1 F4 D3 G6")),
            Subsignal("ba", Pins("F2 F1")),
            Subsignal("dq", Pins("L2 L1 K2 K1 H2 H1 J3 J1 "
                "M3 M1 N2 N1 T2 T1 U2 U1")),
            Subsignal("cke", Pins("H7")),
            Subsignal("we_n", Pins("E3")),
            Subsignal("cs_n", Pins("K6")),  # NC!
            Subsignal("cas_n", Pins("K5")),
            Subsignal("ras_n", Pins("L5")),
            Subsignal("dm", Pins("K3", "K4")),
            Subsignal("dqs", Pins("L4", "P2")),
            Subsignal("rzq", Pins("N4")),
            IOStandard("MOBILE_DDR")),

        # Nat Semi DP83848J 10/100 Ethernet PHY
        # pull-ups on col and rx_data set phy addr to 11111b
        # and prevent isolate mode (addr 00000b)
        ("eth_clocks", 0,
            Subsignal("rx", Pins("L15")),
            Subsignal("tx", Pins("H17")),
            IOStandard("LVCMOS33")),

        ("eth", 0,
            Subsignal("col", Pins("M18"), Misc("PULLUP")),
            Subsignal("crs", Pins("N17"), Misc("PULLDOWN")),
            Subsignal("mdc", Pins("M16"), Misc("PULLDOWN")),
            Subsignal("mdio", Pins("L18"), Misc("PULLUP")),  # 1k5 ext PULLUP
            Subsignal("rst_n", Pins("T18"), Misc("TIG")),
            Subsignal("rx_data", Pins("T17 N16 N15 P18"), Misc("PULLUP")),
            Subsignal("rx_dv", Pins("P17"), Misc("PULLDOWN")),  # MII
            Subsignal("rx_er", Pins("N18"), Misc("PULLUP")),  # auto MDIX
            Subsignal("tx_data", Pins("K18 K17 J18 J16")),
            Subsignal("tx_en", Pins("L17")),
            Subsignal("tx_er", Pins("L16")),  # NC!
            IOStandard("LVCMOS33")),
        ]


class Platform(XilinxPlatform):
    default_clk_name = "clk_y3"
    default_clk_period = 10

    def __init__(self):
        XilinxPlatform.__init__(self, "xc6slx9-2csg324", _io)
        self.add_platform_command("""
CONFIG VCCAUX = "3.3";
""")
        self.toolchain.bitgen_opt = "-g LCK_cycle:6 -g Binary:Yes -w -g SPI_buswidth:4"
        self.toolchain.ise_commands = """
promgen -w -spi -c FF -p mcs -o {build_name}.mcs -u 0 {build_name}.bit
"""

    def do_finalize(self, fragment):
        XilinxPlatform.do_finalize(self, fragment)

        try:
            eth_clocks = self.lookup_request("eth_clocks")
            self.add_period_constraint(eth_clocks.rx, 40)
            self.add_period_constraint(eth_clocks.tx, 40)
            self.add_platform_command("""
TIMESPEC "TS{phy_tx_clk}_io" = FROM "GRP{phy_tx_clk}" TO "PADS" 10 ns;
TIMESPEC "TS{phy_rx_clk}_io" = FROM "PADS" TO "GRP{phy_rx_clk}" 10 ns;
""", phy_rx_clk=eth_clocks.rx, phy_tx_clk=eth_clocks.tx)
        except ConstraintError:
            pass



from migen.build.generic_platform import GenericPlatform
from migen.build.altera import common, quartus


class AlteraPlatform(GenericPlatform):
    bitstream_ext = ".sof"
    create_rbf = True

    def __init__(self, *args, toolchain="quartus", **kwargs):
        GenericPlatform.__init__(self, *args, **kwargs)
        if toolchain == "quartus":
            self.toolchain = quartus.AlteraQuartusToolchain()
        else:
            raise ValueError("Unknown toolchain")

    def get_verilog(self, *args, special_overrides=dict(), **kwargs):
        so = dict(common.altera_special_overrides)
        so.update(special_overrides)
        return GenericPlatform.get_verilog(self, *args, special_overrides=so,
                                           **kwargs)

    def build(self, *args, **kwargs):
        return self.toolchain.build(self, *args, **kwargs)

    def add_period_constraint(self, clk, period):
        if hasattr(clk, "p"):
            clk = clk.p
        self.toolchain.add_period_constraint(self, clk, period)



from migen.build.generic_platform import *
from migen.build.xilinx import XilinxPlatform


_io = [
    ("clk100", 0, Pins("V10"), IOStandard("LVCMOS33")),
    ("clk12", 0, Pins("D9"), IOStandard("LVCMOS33")),

    ("serial", 0,
        Subsignal("tx", Pins("B8"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST")),
        Subsignal("rx", Pins("A8"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST"))),

    ("spiflash", 0,
        Subsignal("cs_n", Pins("V3")),
        Subsignal("clk", Pins("R15")),
        Subsignal("mosi", Pins("T13")),
        Subsignal("miso", Pins("R13"), Misc("PULLUP")),
        IOStandard("LVCMOS33"), Misc("SLEW=FAST")),

    ("ddram_clock", 0,
        Subsignal("p", Pins("G3")),
        Subsignal("n", Pins("G1")),
        IOStandard("MOBILE_DDR")),

    ("ddram", 0,
        Subsignal("a", Pins("J7 J6 H5 L7 F3 H4 H3 H6 D2 D1 F4 D3 G6")),
        Subsignal("ba", Pins("F2 F1")),
        Subsignal("cke", Pins("H7")),
        Subsignal("ras_n", Pins("L5")),
        Subsignal("cas_n", Pins("K5")),
        Subsignal("we_n", Pins("E3")),
        Subsignal(
            "dq", Pins("L2 L1 K2 K1 H2 H1 J3 J1 M3 M1 N2 N1 T2 T1 U2 U1")
        ),
        Subsignal("dqs", Pins("L4 P2")),
        Subsignal("dm", Pins("K3 K4")),
        IOStandard("MOBILE_DDR")),

    ("dipswitch", 0, Pins("C17"), IOStandard("LVCMOS33"), Misc("PULLUP")),
    ("dipswitch", 1, Pins("C18"), IOStandard("LVCMOS33"), Misc("PULLUP")),
    ("dipswitch", 2, Pins("D17"), IOStandard("LVCMOS33"), Misc("PULLUP")),
    ("dipswitch", 3, Pins("D18"), IOStandard("LVCMOS33"), Misc("PULLUP")),
    ("dipswitch", 4, Pins("E18"), IOStandard("LVCMOS33"), Misc("PULLUP")),
    ("dipswitch", 5, Pins("E16"), IOStandard("LVCMOS33"), Misc("PULLUP")),
    ("dipswitch", 6, Pins("F18"), IOStandard("LVCMOS33"), Misc("PULLUP")),
    ("dipswitch", 7, Pins("F17"), IOStandard("LVCMOS33"), Misc("PULLUP")),

    ("buttonswitch", 0, Pins("K18"), IOStandard("LVCMOS33"), Misc("PULLUP")),
    ("buttonswitch", 1, Pins("K17"), IOStandard("LVCMOS33"), Misc("PULLUP")),
    ("buttonswitch", 2, Pins("L17"), IOStandard("LVCMOS33"), Misc("PULLUP")),
    ("buttonswitch", 3, Pins("M16"), IOStandard("LVCMOS33"), Misc("PULLUP")),
    ("buttonswitch", 4, Pins("L18"), IOStandard("LVCMOS33"), Misc("PULLUP")),
    ("buttonswitch", 5, Pins("M18"), IOStandard("LVCMOS33"), Misc("PULLUP")),

    ("user_led", 0, Pins("T18"), IOStandard("LVCMOS33"), Drive(8)),
    ("user_led", 1, Pins("T17"), IOStandard("LVCMOS33"), Drive(8)),
    ("user_led", 2, Pins("U18"), IOStandard("LVCMOS33"), Drive(8)),
    ("user_led", 3, Pins("U17"), IOStandard("LVCMOS33"), Drive(8)),
    ("user_led", 4, Pins("N16"), IOStandard("LVCMOS33"), Drive(8)),
    ("user_led", 5, Pins("N15"), IOStandard("LVCMOS33"), Drive(8)),
    ("user_led", 6, Pins("P16"), IOStandard("LVCMOS33"), Drive(8)),
    ("user_led", 7, Pins("P15"), IOStandard("LVCMOS33"), Drive(8)),

    ("mmc", 0,
        Subsignal("dat", Pins("K14 G18 J13 L13"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST")),

        Subsignal("cmd", Pins("G16"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST")),

        Subsignal("clk", Pins("L12"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST"))),

    ("sevenseg", 0,
        Subsignal("segment7", Pins("A3"), IOStandard("LVCMOS33")),  # A
        Subsignal("segment6", Pins("B4"), IOStandard("LVCMOS33")),  # B
        Subsignal("segment5", Pins("A4"), IOStandard("LVCMOS33")),  # C
        Subsignal("segment4", Pins("C4"), IOStandard("LVCMOS33")),  # D
        Subsignal("segment3", Pins("C5"), IOStandard("LVCMOS33")),  # E
        Subsignal("segment2", Pins("D6"), IOStandard("LVCMOS33")),  # F
        Subsignal("segment1", Pins("C6"), IOStandard("LVCMOS33")),  # G
        Subsignal("segment0", Pins("A5"), IOStandard("LVCMOS33")),  # Dot
        Subsignal("enable0", Pins("B2"), IOStandard("LVCMOS33")),   # EN0
        Subsignal("enable1", Pins("A2"), IOStandard("LVCMOS33")),   # EN1
        Subsignal("enable2", Pins("B3"), IOStandard("LVCMOS33"))),  # EN2


    ("audio", 0,
        Subsignal("channel1", Pins("B16"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST")),
        Subsignal("channel2", Pins("A16"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST"))),

    ("vga_out", 0,
        Subsignal("hsync_n", Pins("B12"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST")),
        Subsignal("vsync_n", Pins("A12"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST")),
        Subsignal("r", Pins("A9 B9 C9"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST")),
        Subsignal("g", Pins("C10 A10 C11"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST")),
        Subsignal("b", Pins("B11 A11"), IOStandard("LVCMOS33"),
                  Misc("SLEW=FAST")))
]

_connectors = [
    ("P6", "T3 R3 V5 U5 V4 T4 V7 U7"),
    ("P7", "V11 U11 V13 U13 T10 R10 T11 R11"),
    ("P8", "L16 L15 K16 K15 J18 J16 H18 H17")
]


class Platform(XilinxPlatform):
    default_clk_name = "clk100"
    default_clk_period = 10

    def __init__(self):
        XilinxPlatform.__init__(self, "xc6slx9-csg324-2", _io, _connectors)

    def create_programmer(self):
        raise NotImplementedError



from migen.build.generic_platform import *
from migen.build.lattice import LatticePlatform
from migen.build.lattice.programmer import IceStormProgrammer


_io = [
    ("user_led_n",    0, Pins("11"), IOStandard("LVCMOS33")),
    ("user_led_n",    1, Pins("37"), IOStandard("LVCMOS33")),
    # Color-specific aliases
    ("user_ledr_n",   0, Pins("11"), IOStandard("LVCMOS33")),
    ("user_ledg_n",   0, Pins("37"), IOStandard("LVCMOS33")),
    ("user_btn_n", 0, Pins("10"), IOStandard("LVCMOS33")),

    ("serial", 0,
        Subsignal("rx", Pins("6")),
        Subsignal("tx", Pins("9"), Misc("PULLUP")),
        IOStandard("LVCMOS33")
    ),

    ("spiflash", 0,
        Subsignal("cs_n",      Pins("16"), IOStandard("LVCMOS33")),
        Subsignal("clk",       Pins("15"), IOStandard("LVCMOS33")),
        Subsignal("miso",        Pins("17"), IOStandard("LVCMOS33")),
        Subsignal("mosi",        Pins("14"), IOStandard("LVCMOS33")),
        Subsignal("wp",      Pins("12"), IOStandard("LVCMOS33")),
        Subsignal("hold", Pins("13"), IOStandard("LVCMOS33")),
    ),

    ("spiflash4x", 0,
        Subsignal("cs_n", Pins("16"), IOStandard("LVCMOS33")),
        Subsignal("clk",  Pins("15"), IOStandard("LVCMOS33")),
        Subsignal("dq",   Pins("14 17 12 13"), IOStandard("LVCMOS33")),
    ),

    ("clk12", 0, Pins("35"), IOStandard("LVCMOS33"))
]

_connectors = [
    ("PMOD1A", "4 2 47 45 3 48 46 44"),
    ("PMOD1B", "43 38 34 31 42 36 32 28"),
    ("PMOD2",  "27 25 21 19 26 23 20 18")
]

# The attached LED/button section can be either used standalone or as a PMOD.
# Attach to platform using:
# plat.add_extension(break_off_pmod)
# pmod_btn = plat.request("user_btn")
break_off_pmod = [
     ("user_btn", 0, Pins("PMOD2:6"), IOStandard("LVCMOS33")),
     ("user_btn", 1, Pins("PMOD2:3"), IOStandard("LVCMOS33")),
     ("user_btn", 2, Pins("PMOD2:7"), IOStandard("LVCMOS33")),

     ("user_led", 0, Pins("PMOD2:4"), IOStandard("LVCMOS33")),
     ("user_led", 1, Pins("PMOD2:0"), IOStandard("LVCMOS33")),
     ("user_led", 2, Pins("PMOD2:1"), IOStandard("LVCMOS33")),
     ("user_led", 3, Pins("PMOD2:5"), IOStandard("LVCMOS33")),
     ("user_led", 4, Pins("PMOD2:2"), IOStandard("LVCMOS33")),

     # Color-specific aliases
     ("user_ledr", 0, Pins("PMOD2:4"), IOStandard("LVCMOS33")),
     ("user_ledg", 0, Pins("PMOD2:0"), IOStandard("LVCMOS33")),
     ("user_ledg", 1, Pins("PMOD2:1"), IOStandard("LVCMOS33")),
     ("user_ledg", 2, Pins("PMOD2:5"), IOStandard("LVCMOS33")),
     ("user_ledg", 3, Pins("PMOD2:2"), IOStandard("LVCMOS33"))
]


class Platform(LatticePlatform):
    default_clk_name = "clk12"
    default_clk_period = 83.333

    def __init__(self):
        LatticePlatform.__init__(self, "ice40-up5k-sg48", _io, _connectors,
                                 toolchain="icestorm")

    def create_programmer(self):
        return IceStormProgrammer()



from migen.build.generic_platform import Pins, IOStandard, Subsignal
from migen.build.altera import AlteraPlatform
from migen.build.altera.programmer import USBBlaster


_io = [
    ("clk1_50", 0, Pins("V11"), IOStandard("3.3-V LVTTL")),
    ("clk2_50", 0, Pins("Y13"), IOStandard("3.3-V LVTTL")),
    ("clk3_50", 0, Pins("E11"), IOStandard("3.3-V LVTTL")),

    ("user_led", 0, Pins("W15"), IOStandard("3.3-V LVTTL")),
    ("user_led", 1, Pins("AA24"), IOStandard("3.3-V LVTTL")),
    ("user_led", 2, Pins("V16"), IOStandard("3.3-V LVTTL")),
    ("user_led", 3, Pins("V15"), IOStandard("3.3-V LVTTL")),
    ("user_led", 4, Pins("AF26"), IOStandard("3.3-V LVTTL")),
    ("user_led", 5, Pins("AE26"), IOStandard("3.3-V LVTTL")),
    ("user_led", 6, Pins("Y16"), IOStandard("3.3-V LVTTL")),
    ("user_led", 7, Pins("AA23"), IOStandard("3.3-V LVTTL")),

    ("key", 0, Pins("AH17"), IOStandard("3.3-V LVTTL")),
    ("key", 1, Pins("AH16"), IOStandard("3.3-V LVTTL")),

    ("sw", 0, Pins("L10"), IOStandard("3.3-V LVTTL")),
    ("sw", 1, Pins("L9"), IOStandard("3.3-V LVTTL")),
    ("sw", 2, Pins("H6"), IOStandard("3.3-V LVTTL")),
    ("sw", 3, Pins("H5"), IOStandard("3.3-V LVTTL")),

    ("epcs", 0,
        Subsignal("data0", Pins("AD7")),
        Subsignal("data1", Pins("AC6")),
        Subsignal("data2", Pins("AC5")),
        Subsignal("data3", Pins("AB6")),
        Subsignal("dclk", Pins("AA8")),
        Subsignal("ncso", Pins("AA6")),
        IOStandard("3.3-V LVTTL")
    ),

    ("adc", 0,
        Subsignal("convst", Pins("U9")),
        Subsignal("sck", Pins("V10")),
        Subsignal("sdi", Pins("AC4")),
        Subsignal("sdo", Pins("AD4")),
        IOStandard("3.3-V LVTTL")
    ),

    ("gpio_0", 0,
        Pins("V12 AF7 W12 AF8 Y8 AB4 W8 Y4 Y5 U11 T8 T12 AH5 AH6 AH4 AG5 AH3",
             "AH2 AF4 AG6 AF5 AE4 T13 T11 AE7 AF6 AF9 AE8 AD10 AE9 AD11 AF10",
             "AD12 AE11 AF11 AE12"),
        IOStandard("3.3-V LVTTL")
    ),

    ("gpio_1", 0,
        Pins("Y15 AG28 AA15 AH27 AG26 AH24 AF23 AE22 AF21 AG20 AG19 AF20",
             "AC23 AG18 AH26 AA19 AG24 AF25 AH23 AG23 AE19 AF18 AD19 AE20",
             "AE24 AD20 AF22 AH22 AH19 AH21 AG21 AH18 AD23 AE23 AA18 AC22"),
        IOStandard("3.3-V LVTTL")
    ),

    ("arduino", 0,
        Subsignal("gpio",
            Pins("AG13 AF13 AG10 AG9 U14 U13 AG8 AH8 AF17 AE15 AF15 AG16 AH11",
                 "AH12 AH9 AG11")),
        Subsignal("reset", Pins("AH7")),
        IOStandard("3.3-V LVTTL")
    ),

    ("hps", 0,
        Subsignal("key", Pins("J18"), IOStandard("3.3-V LVTTL")),
        Subsignal("led", Pins("A20"), IOStandard("3.3-V LVTTL")),

        Subsignal("eth",
            Subsignal("tx_en", Pins("A12")),
            Subsignal("tx_data", Pins("A16 J14 A15 D17")),
            Subsignal("rx_dv", Pins("J13")),
            Subsignal("rx_data", Pins("A14 A11 C15 A9")),
            Subsignal("reset_n", Pins("B4")),
            Subsignal("mdio", Pins("E16")),
            Subsignal("mdc", Pins("A13")),
            Subsignal("int_n", Pins("B14")),
            IOStandard("3.3-V LVTTL")
        ),

        Subsignal("eth_clocks",
            Subsignal("rx", Pins("J12")),
            Subsignal("gtx", Pins("J15")),
            IOStandard("3.3-V LVTTL")
        ),

        Subsignal("serial",
            Subsignal("rx", Pins("A22")),
            Subsignal("tx", Pins("B21")),
            Subsignal("conv_usb_n", Pins("C6")),
            IOStandard("3.3-V LVTTL")
        ),

        Subsignal("sd",
            Subsignal("clk", Pins("B8")),
            Subsignal("cmd", Pins("D14")),
            Subsignal("data", Pins("C13 B6 B11 B9")),
            IOStandard("3.3-V LVTTL")
        ),

        Subsignal("usb",
            Subsignal("clkout", Pins("G4")),
            Subsignal("data", Pins("C10 F5 C9 C4 C8 D4 C7 F4")),
            Subsignal("dir", Pins("E5")),
            Subsignal("nxt", Pins("D5")),
            Subsignal("reset", Pins("H12")),
            Subsignal("stp", Pins("C5")),
            IOStandard("3.3-V LVTTL")
        ),

        Subsignal("gsensor_int", Pins("A17"), IOStandard("3.3-V LVTTL")),
        Subsignal("i2c0",
            Subsignal("sclk", Pins("C18")),
            Subsignal("sdat", Pins("A19")),
            IOStandard("3.3-V LVTTL")
        ),

        Subsignal("ltc_gpio", Pins("H13"), IOStandard("3.3-V LVTTL")),
        Subsignal("i2c1",
            Subsignal("sclk", Pins("B21")),
            Subsignal("sdat", Pins("A21")),
            IOStandard("3.3-V LVTTL")
        ),
        Subsignal("spim",
            Subsignal("clk", Pins("C19")),
            Subsignal("miso", Pins("B19")),
            Subsignal("mosi", Pins("B16")),
            Subsignal("ss", Pins("C16")),
            IOStandard("3.3-V LVTTL")
        ),

        Subsignal("ddram",
            Subsignal("a",
                      Pins("C28 B28 E26 D26 J21 J20 C26 B26",
                           "F26 F25 A24 B24 D24 C24 G23"),
                      IOStandard("SSTL-15 Class I")),
            Subsignal("ba", Pins("A27 H25 G25"),
                      IOStandard("SSTL-15 Class I")),
            Subsignal("cas_n", Pins("A26"),
                      IOStandard("SSTL-15 Class I")),
            Subsignal("cke", Pins("L28"),
                      IOStandard("SSTL-15 Class I")),
            Subsignal("ck_n", Pins("N20"),
                      IOStandard("Differential 1.5-V SSTL Class I")),
            Subsignal("ck_p", Pins("N21"),
                      IOStandard("Differential 1.5-V SSTL Class I")),
            Subsignal("cs_n", Pins("L21"),
                      IOStandard("SSTL-15 Class I")),
            Subsignal("dm", Pins("G28 P28 W28 AB28"),
                      IOStandard("SSTL-15 Class I")),
            Subsignal("dq",
                      Pins("J25 J24 E28 D27 J26 K26 G27 F28 K25 L25 J27",
                           "J28 M27 M26 M28 N28 N24 N25 T28 U28 N26 N27",
                           "R27 V27 R26 R25 AA28 W26 R24 T24 Y27 AA27"),
                      IOStandard("SSTL-15 Class I")),
            Subsignal("dqs_n", Pins("R16 R18 T18 T20"),
                      IOStandard("Differential 1.5-V SSTL Class I")),
            Subsignal("dqs_p", Pins("R17 R19 T19 U19"),
                      IOStandard("Differential 1.5-V SSTL Class I")),
            Subsignal("odt", Pins("D28"), IOStandard("SSTL-15 Class I")),
            Subsignal("ras_n", Pins("A25"), IOStandard("SSTL-15 Class I")),
            Subsignal("reset_n", Pins("V28"), IOStandard("SSTL-15 Class I")),
            Subsignal("we_n", Pins("E25"), IOStandard("SSTL-15 Class I")),
            Subsignal("rzq", Pins("D25"), IOStandard("SSTL-15 Class I"))),
    ),
]


class Platform(AlteraPlatform):
    default_clk_name = "clk1_50"
    default_clk_period = 20

    def __init__(self):
        AlteraPlatform.__init__(self, "5CSEMA4U23C6", _io)

    def create_programmer(self):
        return USBBlaster(cable_name="DE-SoC", device_id=2)



import collections.abc
from itertools import combinations

from migen.util.misc import flat_iteration
from migen.fhdl.structure import *
from migen.fhdl.structure import _Fragment
from migen.fhdl.tools import rename_clock_domain


__all__ = ["Module", "FinalizeError"]


class FinalizeError(Exception):
    pass


def _flat_list(e):
    if isinstance(e, collections.abc.Iterable):
        return flat_iteration(e)
    else:
        return [e]


class _ModuleProxy:
    def __init__(self, fm):
        object.__setattr__(self, "_fm", fm)


class _ModuleComb(_ModuleProxy):
    def __iadd__(self, other):
        self._fm._fragment.comb += _flat_list(other)
        return self


def _cd_append(d, key, statements):
    try:
        l = d[key]
    except KeyError:
        l = []
        d[key] = l
    l += _flat_list(statements)


class _ModuleSyncCD:
    def __init__(self, fm, cd):
        self._fm = fm
        self._cd = cd

    def __iadd__(self, other):
        _cd_append(self._fm._fragment.sync, self._cd, other)
        return self


class _ModuleSync(_ModuleProxy):
    def __iadd__(self, other):
        _cd_append(self._fm._fragment.sync, "sys", other)
        return self

    def __getattr__(self, name):
        return _ModuleSyncCD(self._fm, name)

    def __setattr__(self, name, value):
        if not isinstance(value, _ModuleSyncCD):
            raise AttributeError("Attempted to assign sync property - use += instead")


# _ModuleForwardAttr enables user classes to do e.g.:
# self.subm.foobar = SomeModule()
# and then access the submodule with self.foobar.
class _ModuleForwardAttr:
    def __setattr__(self, name, value):
        self.__iadd__(value)
        setattr(self._fm, name, value)


class _ModuleSpecials(_ModuleProxy, _ModuleForwardAttr):
    def __iadd__(self, other):
        self._fm._fragment.specials |= set(_flat_list(other))
        return self


class _ModuleSubmodules(_ModuleProxy):
    def __setattr__(self, name, value):
        self._fm._submodules += [(name, e) for e in _flat_list(value)]
        setattr(self._fm, name, value)

    def __iadd__(self, other):
        self._fm._submodules += [(None, e) for e in _flat_list(other)]
        return self


class _ModuleClockDomains(_ModuleProxy, _ModuleForwardAttr):
    def __iadd__(self, other):
        self._fm._fragment.clock_domains += _flat_list(other)
        return self


class Module:
    def get_fragment(self):
        assert(not self.get_fragment_called)
        self.get_fragment_called = True
        self.finalize()
        return self._fragment

    def __getattr__(self, name):
        if name == "comb":
            return _ModuleComb(self)
        elif name == "sync":
            return _ModuleSync(self)
        elif name == "specials":
            return _ModuleSpecials(self)
        elif name == "submodules":
            return _ModuleSubmodules(self)
        elif name == "clock_domains":
            return _ModuleClockDomains(self)

        # hack to have initialized regular attributes without using __init__
        # (which would require derived classes to call it)
        elif name == "finalized":
            self.finalized = False
            return self.finalized
        elif name == "_fragment":
            self._fragment = _Fragment()
            return self._fragment
        elif name == "_submodules":
            self._submodules = []
            return self._submodules
        elif name == "_clock_domains":
            self._clock_domains = []
            return self._clock_domains
        elif name == "get_fragment_called":
            self.get_fragment_called = False
            return self.get_fragment_called

        else:
            raise AttributeError("'"+self.__class__.__name__+"' object has no attribute '"+name+"'")

    def __setattr__(self, name, value):
        if name in ["comb", "sync", "specials", "submodules", "clock_domains"]:
            if not isinstance(value, _ModuleProxy):
                raise AttributeError("Attempted to assign special Module property - use += instead")
        else:
            object.__setattr__(self, name, value)

    def _collect_submodules(self):
        r = []
        for name, submodule in self._submodules:
            if not submodule.get_fragment_called:
                r.append((name, submodule.get_fragment()))
        return r

    def finalize(self, *args, **kwargs):
        if not self.finalized:
            self.finalized = True
            # finalize existing submodules before finalizing us
            subfragments = self._collect_submodules()
            self.do_finalize(*args, **kwargs)
            # finalize submodules created by do_finalize
            subfragments += self._collect_submodules()
            # resolve clock domain name conflicts
            needs_renaming = set()
            for (mod_name1, f1), (mod_name2, f2) in combinations(subfragments, 2):
                f1_names = set(cd.name for cd in f1.clock_domains)
                f2_names = set(cd.name for cd in f2.clock_domains)
                common_names = f1_names & f2_names
                if common_names:
                    if mod_name1 is None or mod_name2 is None:
                        raise ValueError("Multiple submodules with local clock domains cannot be anonymous")
                    if mod_name1 == mod_name2:
                        raise ValueError("Multiple submodules with local clock domains cannot have the same name")
                needs_renaming |= common_names
            for mod_name, f in subfragments:
                for cd in f.clock_domains:
                    if cd.name in needs_renaming:
                        rename_clock_domain(f, cd.name, mod_name + "_" + cd.name)
            # sum subfragments
            for mod_name, f in subfragments:
                self._fragment += f

    def do_finalize(self):
        pass

    def do_exit(self, *args, **kwargs):
        for name, submodule in self._submodules:
            submodule.do_exit(*args, **kwargs)



from migen.fhdl.structure import *
from migen.fhdl.module import Module


(SP_WITHDRAW, SP_CE) = range(2)


class RoundRobin(Module):
    def __init__(self, n, switch_policy=SP_WITHDRAW):
        self.request = Signal(n)
        self.grant = Signal(max=max(2, n))
        self.switch_policy = switch_policy
        if self.switch_policy == SP_CE:
            self.ce = Signal()

        ###

        if n > 1:
            cases = {}
            for i in range(n):
                switch = []
                for j in reversed(range(i+1, i+n)):
                    t = j % n
                    switch = [
                        If(self.request[t],
                            self.grant.eq(t)
                        ).Else(
                            *switch
                        )
                    ]
                if self.switch_policy == SP_WITHDRAW:
                    case = [If(~self.request[i], *switch)]
                else:
                    case = switch
                cases[i] = case
            statement = Case(self.grant, cases)
            if self.switch_policy == SP_CE:
                statement = If(self.ce, statement)
            self.sync += statement
        else:
            self.comb += self.grant.eq(0)



from migen.fhdl.structure import *
from migen.fhdl.module import Module


class BitonicSort(Module):
    """Combinatorial sorting network

    The Bitonic sort is implemented as a combinatorial sort using
    comparators and multiplexers. Its asymptotic complexity (in terms of
    number of comparators/muxes) is O(n log(n)**2), like mergesort or
    shellsort.

    http://www.dps.uibk.ac.at/~cosenza/teaching/gpu/sort-batcher.pdf

    http://www.inf.fh-flensburg.de/lang/algorithmen/sortieren/bitonic/bitonicen.htm

    http://www.myhdl.org/doku.php/cookbook:bitonic

    Parameters
    ----------
    n : int
        Number of inputs and output signals.
    m : int
        Bit width of inputs and outputs. Or a tuple of `(m, signed)`.
    ascending : bool
        Sort direction. `True` if input is to be sorted ascending,
        `False` for descending. Defaults to ascending.

    Attributes
    ----------
    i : list of Signals, in
        Input values, each `m` wide.
    o : list of Signals, out
        Output values, sorted, each `m` bits wide.
    """
    def __init__(self, n, m, ascending=True):
        self.i = [Signal(m) for i in range(n)]
        self.o = [Signal(m) for i in range(n)]
        self._sort(self.i, self.o, int(ascending), m)

    def _sort_two(self, i0, i1, o0, o1, dir):
        self.comb += [
                o0.eq(i0),
                o1.eq(i1),
                If(dir == (i0 > i1),
                    o0.eq(i1),
                    o1.eq(i0),
                )]

    def _merge(self, i, o, dir, m):
        n = len(i)
        k = n//2
        if n > 1:
            t = [Signal(m) for j in range(n)]
            for j in range(k):
                self._sort_two(i[j], i[j + k], t[j], t[j + k], dir)
            self._merge(t[:k], o[:k], dir, m)
            self._merge(t[k:], o[k:], dir, m)
        else:
            self.comb += o[0].eq(i[0])

    def _sort(self, i, o, dir, m):
        n = len(i)
        k = n//2
        if n > 1:
            t = [Signal(m) for j in range(n)]
            self._sort(i[:k], t[:k], 1, m)  # ascending
            self._sort(i[k:], t[k:], 0, m)  # descending
            self._merge(t, o, dir, m)
        else:
            self.comb += o[0].eq(i[0])
